# PaymentOrchestration engine

Short description and motivation.

## Usage
How to use this plugin.

## Installation
Add this line to your application's Gemfile:

```ruby
gem "payment_orchestration"
```

And then execute:
```bash
$ bundle
```

Or install it yourself as:
```bash
$ gem install payment_orchestration
```

## Contributing
Contribution directions go here.

## Setup notes

``` bash
rails plugin new payment_orchestration --mountable -d sqlite3 -T --dummy-path=spec/dummy --skip-bundle
```

Add the following to the gemspec file:

``` ruby
  spec.files = Dir.chdir(File.expand_path(__dir__)) do
    Dir["{app,bin,config,db,lib}/**/*", "MIT-LICENSE", "Rakefile", "README.md"] +
    Dir["spec/dummy/**/*"] +
    ["#{spec.name}.gemspec", "Rakefile"]
  end

  # spec.test_files = Dir["{test,spec,features}/**/*"]
  spec.require_paths = ["lib"]

  spec.rdoc_options = ["--line-numbers", "--inline-source", "--title", "payment_orchestration", "--main", "README.rdoc"]
  spec.extra_rdoc_files = ["CHANGELOG", "README.rdoc", "lib/tasks/payment_orchestration_tasks.rake", "lib/payment_orchestration.rb"]
```

### Get basic gem generation working

``` bash
rake build
```

### RSpec

``` bash
bundle exec rails generate rspec:install
```

#### Tweaks for  'rails_helper.rb'

``` ruby
# This file is copied to spec/ when you run 'rails generate rspec:install'
require 'spec_helper'

ENV['RAILS_ENV'] ||= 'test'
require_relative '../config/environment'
require_relative '../spec/dummy/config/environment'

# Prevent database truncation if the environment is production
abort("The Rails environment is running in production mode!") if Rails.env.production?

# Uncomment the line below in case you have `--require rails_helper` in the `.rspec` file
# that will avoid rails generators crashing because migrations haven't been run yet
# return unless Rails.env.test?

require 'rspec/rails'

# Add additional requires below this line. Rails is not loaded until this point!

require 'factory_bot_rails'
require 'database_cleaner'
require 'shoulda/matchers/integrations/rspec'

ENGINE_RAILS_ROOT=File.join(File.dirname(__FILE__), '../')

# Requires supporting ruby files with custom matchers and macros, etc,
# in spec/support/ and its subdirectories.
# Dir[Rails.root.join("spec/support/**/*.rb")].each {|f| require f}
Dir[PaymentOrchestration::Engine.root.join('spec/support/**/*.rb')].each {|f| require f }

# Requires supporting ruby files with custom matchers and macros, etc, in
# spec/support/ and its subdirectories. Files matching `spec/**/*_spec.rb` are
# run as spec files by default. This means that files in spec/support that end
# in _spec.rb will both be required and run as specs, causing the specs to be
# run twice. It is recommended that you do not name files matching this glob to
# end with _spec.rb. You can configure this pattern with the --pattern
# option on the command line or in ~/.rspec, .rspec or `.rspec-local`.
#
# The following line is provided for convenience purposes. It has the downside
# of increasing the boot-up time by auto-requiring all files in the support
# directory. Alternatively, in the individual `*_spec.rb` files, manually
# require only the support files necessary.
#
# Rails.root.glob('spec/support/**/*.rb').sort_by(&:to_s).each { |f| require f }

# Ensures that the test database schema matches the current schema file.
# If there are pending migrations it will invoke `db:test:prepare` to
# recreate the test database by loading the schema.
# If you are not using ActiveRecord, you can remove these lines.
begin
  ActiveRecord::Migration.maintain_test_schema!
rescue ActiveRecord::PendingMigrationError => e
  abort e.to_s.strip
end

RSpec.configure do |config|
  # ## Mock Framework
  #
  # If you prefer to use mocha, flexmock or RR, uncomment the appropriate line:
  #
  # config.mock_with :mocha
  # config.mock_with :flexmock
  # config.mock_with :rr
  config.mock_with :rspec

  # Remove this line if you're not using ActiveRecord or ActiveRecord fixtures
  config.fixture_paths = [
    Rails.root.join('spec/fixtures')
  ]

  # If you're not using ActiveRecord, or you'd prefer not to run each of your
  # examples within a transaction, remove the following line or assign false
  # instead of true.
  config.use_transactional_fixtures = true

  # Use DatabaseCleaner to manage transactions
  config.before(:suite) do
    DatabaseCleaner.strategy = :transaction
    DatabaseCleaner.clean_with(:truncation)
  end
  config.before(:each) do
    DatabaseCleaner.start
  end
  config.after(:each) do
    DatabaseCleaner.clean
  end

  # You can uncomment this line to turn off ActiveRecord support entirely.
  # config.use_active_record = false

  # RSpec Rails uses metadata to mix in different behaviours to your tests,
  # for example enabling you to call `get` and `post` in request specs. e.g.:
  #
  #     RSpec.describe UsersController, type: :request do
  #       # ...
  #     end
  #
  # The different available types are documented in the features, such as in
  # https://rspec.info/features/8-0/rspec-rails
  #
  # You can also this infer these behaviours automatically by location, e.g.
  # /spec/models would pull in the same behaviour as `type: :model` but this
  # behaviour is considered legacy and will be removed in a future version.
  #
  # To enable this behaviour uncomment the line below.
  # config.infer_spec_type_from_file_location!

  # Filter lines from Rails gems in backtraces.
  config.filter_rails_from_backtrace!
  # arbitrary gems may also be filtered via:
  # config.filter_gems_from_backtrace("gem name")

  # Include Engine routes (needed for Controller specs)
  config.include PaymentOrchestration::Engine.routes.url_helpers
```

### Test RSpec will run:

``` bash
bundle exec rspec
```

### Cucumber

bundle exec rails generate cucumber:install --rspec --capybara

#### Tweaks for 'env.rb'

Add this to the end of the file

``` ruby
# include Engine routes in Cucumber world
module EngineRoutesHelper
  include PaymentOrchestration::Engine.routes.url_helpers
end
World(EngineRoutesHelper)

```

### Test Cucumber will run:

``` bash
bundle exec cucumber RAILS_ROOT=spec/dummy
```

### Generate Viwes and scaffolds for Model definitions

``` bash
bundle exec rails generate controller Dashboard index

bundle exec rails generate scaffold Payment description:text payment_method:string status:string
bundle exec rails generate scaffold PaymentDetails payment:references invoice_reference:string items:text tax:decimal total_amount:decimal currency:references
bundle exec rails generate scaffold Transaction payment:references transaction_reference_code:string transaction_result:text

bundle exec rails generate controller Orchestrator get_payment_method get_details_for_payment_method show_transaction_status
```

Then, add the following to config/routes.rb:

``` ruby
  root to: "dashboard#index"
```

## Development notes

### Development with docker-compose

``` bash
(docker-compose build --build-arg BUILD_ENV=local && docker-compose --project-name microservices up && docker-compose --project-name microservices down)
```

### Deployment to kubernetes

#### Create kubernetes namespace

``` bash
(kubectl get namespace "microservices" || kubectl create namespace "microservices")
kubectl label namespace microservices istio-injection=enabled
```

#### Create regcred

``` bash
kubectl --namespace="microservices" delete secret regcred
```

``` bash
kubectl create secret docker-registry regcred \
  --namespace="microservices" \
  --docker-server=docker-registry.localnet.htcl.eu \
  --docker-email=name@domain.com \
  --docker-username=everard \
  --docker-password=3...l
```

``` bash
kubectl create secret docker-registry regcred \
  --namespace="microservices" \
  --docker-server=artifactory.localnet.htcl.eu:5001 \
  --docker-email=name@domain.com \
  --docker-username=everard \
  --docker-password=3...L
```

#### Update/Fix nginx-ingress-controller configuration

##### RKE

``` bash
kubectl apply -f kubernetes/rke-nginx-configuration.yaml
```

##### RKE2

``` bash
kubectl apply -f kubernetes/rke2-nginx-configuration.yaml
```

#### Image creation/deployment

``` bash
time docker-compose build --build-arg BUILD_ENV=local --no-cache --pull
  OR
time docker-compose build --build-arg BUILD_ENV=local --build-arg TARGET_ENVIRONMENT=':localhost'
  OR
time docker-compose build --build-arg BUILD_ENV=local --build-arg TARGET_ENVIRONMENT=':k8s-localhost'
  OR
time docker-compose build --build-arg BUILD_ENV=local --build-arg RAILS_ENV=production
```

  OR

``` bash
time docker build --build-arg BUILD_ENV=local --no-cache --pull --tag payment-orchestration .
  OR
time docker build --build-arg BUILD_ENV=local ---build-arg TARGET_ENVIRONMENT=':localhost' -tag payment-orchestration .
  OR
time docker build --build-arg BUILD_ENV=local --build-arg TARGET_ENVIRONMENT=':k8s-localhost' --tag payment-orchestration .
  OR
time docker build --build-arg BUILD_ENV=local --build-arg RAILS_ENV=production --tag payment-orchestration .
```

``` bash
# image="docker-registry.localnet.htcl.eu/payment-orchestration:latest"
image="docker-registry.localnet.htcl.eu/payment-orchestration:$(date +%Y%m%d%H%M%S)_$(git --no-pager log -1 --pretty=%h)" ; \
latest_image="docker-registry.localnet.htcl.eu/payment-orchestration:latest" && \
docker tag payment-orchestration $image && docker tag payment-orchestration $latest_image && \
docker push $image && docker push $latest_image && \
docker rmi $image && docker rmi $latest_image
```

#### Application deployment

``` bash
export DATABASE_NAME="$(basename $(pwd) | sed 's|-|_|g')_engine_db"
# export DATABASE_HOSTNAME=postgresql.default.svc.cluster.local
export DATABASE_HOSTNAME=postgresql.default
export DATABASE_USER=postgres
export DATABASE_PASSWORD=$(kubectl get secret --namespace default postgresql -o jsonpath="{.data.postgres-password}" | base64 -d)
```

``` bash
./kubernetes/deploy --namespace "microservices" --image $image --ingress payment-orchestration.localnet.htcl.eu,payment-orchestration.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --include 'payment-orchestration|persistentvolumeclaim' --exclude 'postgres|pgadmin' --env-file .env.k8s-localhost --verbose

./kubernetes/deploy --namespace "microservices" --image $image --ingress payment-orchestration.localnet.htcl.eu,payment-orchestration.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --include 'payment-orchestration|persistentvolumeclaim' --env-file .env.k8s-localhost --verbose

./kubernetes/deploy --namespace "microservices" --image $image --ingress payment-orchestration.localnet.htcl.eu,payment-orchestration.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --verbose

./kubernetes/deploy --namespace "microservices" --image $image --ingress payment-orchestration.localnet.htcl.eu,payment-orchestration.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --k8s-server k8s-localhost-centos8.localnet.htcl.eu --k8s-port 6443 --verbose

./kubernetes/deploy --namespace "microservices"  --image $image --ingress payment-orchestration.localnet.htcl.eu,payment-orchestration.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --verbose --redeploy --dry-run

./kubernetes/deploy --namespace "microservices" --image $image --ingress payment-orchestration.localnet.htcl.eu,payment-orchestration.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --rails-env production --verbose
```

Spare parts

``` bash
 --redeploy
 --dry-run

 --replicas POD_REPLICAS
 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG
 --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG

 --pvc-storageclassname "default"
 --pvc-storageclassname "local-path"
 --pvc-storageclassname "longhorn"
```

#### Check deployment

``` bash
helm list --all --namespace "microservices"
helm list --all --all-namespaces
```

## License
The gem is available as open source under the terms of the [MIT License](https://opensource.org/licenses/MIT).
