require "faker"

module PaymentOrchestration
  class OrchestratorController < ApplicationController
    def start_payment
      country_code, locale, currency_code =
      [
        %w[DE de EUR],
        %w[GB en-GB GBP],
        %w[ES es EUR],
        %w[FR fr EUR],
        %w[IL he ILS],
        %w[IN en-IND INR],
        %w[SA ar SAR],
        %w[CN zh-CN CNY],
        # %w[RO ro RON],
        %w[JP ja JPY],
        # %w[UA uk_UA UAH],
        %w[RU ru RUB],
        # %w[MA fr_MA MAD]
      ].sample

      Faker::Config.locale = locale


      payment_datails ||= session[:payment_details]
      payment_datails ||= params[:payment_details]
      unless payment_datails
        currency = CoreModels::Base::Currency.find_by(iso_code: currency_code)
        items = []

        rand(10).times do
          items.push({
            quantity: rand(1..10),
            item: Faker::Commerce.product_name,
            unit_price: Faker::Commerce.price
          })
        end

        tax = rand(5.0..25.0).round(1)
        net_price = (items.sum { |h| h[:unit_price] }).round(2)
        tax_price = (net_price * (tax/100)).round(2)


        payment_datails = {
          reference_number: SecureRandom.hex(10),
          currency: currency.iso_code,
          items: items,
          tax: tax,
          net_price: net_price,
          tax_price: tax_price,
          total_price: net_price + tax_price
        }
      end

      delivery_details ||= session[:delivery_details]
      delivery_details ||= params[:delivery_details]
      unless delivery_details
        country = CoreModels::I18n::Country.find_by(iso_code: country_code)

        address = CoreModels::Base::Address.new(
          address_line_1: Faker::Address.street_address,
          city: country.cities.sample,
          postal_code: Faker::Address.postcode,
          country: country
        )

        delivery_details = {
          name: "#{Faker::Name.neutral_first_name} #{Faker::Name.last_name.upcase}",
          address: address.to_s
        }
      end

      @payment = PaymentOrchestration::Payment.new(payment_details: payment_datails, delivery_details: delivery_details)

      session[:payment] = @payment # .as_json(root: true)
      redirect_to controller: :orchestrator, action: :get_payment_method
    end

    def get_payment_method
      @payment = PaymentOrchestration::Payment.new(payment_params)
    end

    def got_payment_method
      # Go back if payment_method not set
      unless params[:payment_method].present?
        flash[:alert] = "Please select a payment method"
        redirect_to controller: :orchestrator, action: :get_payment_method
        return
      end

      @payment = PaymentOrchestration::Payment.new(payment_params)

      # Process step
      @payment.payment_method = params[:payment_method] if params[:payment_method]

      session[:payment] = @payment # .as_json(root: true)
      redirect_to controller: :orchestrator, action: :get_details_for_payment_method
    end

    def get_details_for_payment_method
      @payment = PaymentOrchestration::Payment.new(payment_params)
    end

    def got_details_for_payment_method
      @payment = PaymentOrchestration::Payment.new(payment_params)

      # Initiate payment

      session[:payment] = @payment # .as_json(root: true)
      redirect_to controller: :orchestrator, action: :show_transaction_status
    end

    def show_transaction_status
      @payment = PaymentOrchestration::Payment.new(payment_params)
    end

    private
      # Only allow a list of trusted parameters through.
      def payment_params
        # params.expect(payment: [ :description, :payment_method, :status, payment_details: [], delivery_details: [] ])

        # params.require(:payment).permit(
        #   :description, :payment_method, :status,
        #   payment_details: [],
        #   delivery_details: []
        # )

        session[:payment]
      end
  end
end
